@@ -15,18 +15,28 @@ import json
 import datetime
 
 
+"""
+Kraken_db api
+
+methods:
+
+- get:
+    - input:
+
+- search:
+
+
 
 """
-Init db engine
-"""
-#engine = create_engine('sqlite:///:memory:', echo=False)
+
+
+
+
 engine = create_engine('sqlite:///test.sqlite?''uri=true&check_same_thread=False&timeout=10', echo=False)
 
-
-
-
 Base = declarative_base()
 
+
 Session = sessionmaker(bind=engine)
 session = Session()
 
@@ -39,8 +49,8 @@ Class declaration
 class Observation(Base):
     __tablename__ = 'observations'
 
-    id = Column(Integer, primary_key=True)
-    observation_id = Column(String)
+    id = Column(String)
+    observation_id = Column(String, primary_key=True)
     ref_id = Column(String)
     datasource = Column(String)
     agent = Column(String)
@@ -72,9 +82,18 @@ Base.metadata.create_all(engine)
 
 
 """
-API
+API - observations
 """
 
+
+def rollback():
+    session.rollback()
+
+
+
+
+
+
 def get(record_type = None, record_id = None, key = None, value = None):
     # Get observations matching conditions
     query = _get_observations(record_type, record_id, key, value)
@@ -109,6 +128,12 @@ def get_summary_by_value(record_type = None, record_id = None):
     return _to_dict(query)
 
 
+
+"""
+API - entities
+"""
+
+
 def search(params, order_by = None, order_direction = None, limit = None, offset = None):
 
     query = _search(params, order_by, order_direction, limit, offset)
@@ -377,13 +402,17 @@ def _search(params, order_by = None, order_direction = None, limit = None, offse
 
     # Step 1. Find observations meeting conditions
     for key, operator, value in params:
-
-        sub_query = session.query(obs0)
+        sub_query = session.query(obs0.ref_id, obs0.record_id)
 
         if key == 'record_type':
             sub_query = sub_query.filter(obs0.record_type.like(value))
         elif key == 'record_id':
             sub_query = sub_query.filter(obs0.record_id.like(value))
+        
+        elif key == 'created_date':
+            sub_query = sub_query.filter(obs0.created_date.like(value))        
+        
+        
         else:
             sub_query = sub_query.filter(obs0.key.like(key))
             sub_query = sub_query.filter(obs0.value.like(value))
@@ -394,14 +423,16 @@ def _search(params, order_by = None, order_direction = None, limit = None, offse
             query = sub_query
 
 
-    # Step 2. Retrieve all observations with same ref_id
-    obs1 = aliased(Observation)
-    query = query.join(obs1, obs0.ref_id == obs1.ref_id)
-
-
     # apply order_by
     if order_by:
-        query = query.order_by(order_by)
+        if order_direction not in ['asc', 'desc']:
+            order_direction = 'asc'
+
+        query = query.order_by(getattr(getattr(obs0, order_by), order_direction)())
+
+
+    # Apply unique
+    query = query.distinct(obs0.ref_id)
 
 
     # apply limit and offset
@@ -410,9 +441,20 @@ def _search(params, order_by = None, order_direction = None, limit = None, offse
     if offset:
         query = query.offset(offset)
 
+    query = query.subquery()
 
 
-    return query
+
+    # Step 2. Retrieve all observations with same ref_id
+    #obs1 = aliased(Observation)
+    subquery1 = session.query(Observation)
+    subquery1 = subquery1.join(query, query.c.ref_id == Observation.ref_id)
+
+
+    
+
+
+    return subquery1
 
 
 
